@@ -183,8 +183,6 @@ void parentname2(char* result, int result_size, char* path)
 
 migemo* gMigemo;
 
-hash_obj* gMigemoRegsCache;
-
 void migemo_init()
 {
     char buf[PATH_MAX];
@@ -259,19 +257,10 @@ void migemo_init()
             exit(1);
         }
     }
-    gMigemoRegsCache = HASH_NEW(100);
 }
 
 void migemo_final()
 {
-    hash_it* it = hash_loop_begin(gMigemoRegsCache);
-    while(it) {
-       regex_t* obj = hash_loop_item(it);
-       onig_free(obj);
- 
-       it = hash_loop_next(it);
-    }
-    hash_delete(gMigemoRegsCache);
     onig_end();
     migemo_close(gMigemo);
 }
@@ -341,39 +330,20 @@ char* readline_file_completion_migemo_match(char* text, char* file, regex_t* reg
         *_p2 = 0;
 
         regex_t* reg;
-        if(strlen(p2) <= 2) {
-            reg = hash_item(gMigemoRegsCache, p2);
-
-            if(reg == NULL) {
-                if(gKanjiCode == kUtf8) {
-                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
-                }
-                else if(gKanjiCode == kEucjp) {
-                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
-                }
-                else if(gKanjiCode == kSjis) {
-                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
-                }
-
-                hash_put(gMigemoRegsCache, p2, reg);
-            }
+        if(gKanjiCode == kUtf8) {
+            r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
         }
-        else { 
-            if(gKanjiCode == kUtf8) {
-                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
-            }
-            else if(gKanjiCode == kEucjp) {
-                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
-            }
-            else if(gKanjiCode == kSjis) {
-                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
-            }
+        else if(gKanjiCode == kEucjp) {
+            r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
+        }
+        else if(gKanjiCode == kSjis) {
+            r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
         }
 
         migemo_release(gMigemo, (unsigned char*) p);
 
         if(r != ONIG_NORMAL) {
-            if(strlen(p2) > 2) { onig_free(reg); }
+            if(strlen(p2) > 2) { puts("AAA"); onig_free(reg); }
 
             FREE(p2);
             return NULL;
@@ -386,7 +356,7 @@ char* readline_file_completion_migemo_match(char* text, char* file, regex_t* reg
 
         onig_region_free(region, 1);
 
-        if(strlen(p2) > 2) { onig_free(reg); }
+        if(strlen(p2) > 2) { puts("BBB"); onig_free(reg); }
 
         FREE(p2);
     }
@@ -400,6 +370,7 @@ char* readline_file_completion_migemo_match(char* text, char* file, regex_t* reg
     return NULL;
 }
 
+static regex_t* gMigemoReg = NULL;
 
 char* readline_file_completion_migemo(const char* text, int rstat)
 {
@@ -419,7 +390,6 @@ char* readline_file_completion_migemo(const char* text, int rstat)
     *_p2 = 0;
 
     static int index, wordlen;
-    static regex_t* reg = NULL;
 
     /// 初期化 ///
     if(rstat == 0) {
@@ -427,8 +397,6 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         index = 0;
 
         /// migemoのクェリーと正規表現コンパイル結果を得る
-        if(reg) { if(hash_key(gMigemoRegsCache, reg) == NULL) { onig_free(reg); reg = NULL; } }
-
         OnigUChar* p = migemo_query(gMigemo, text2);
 
         if(p == NULL) {
@@ -455,33 +423,14 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         *_p2 = 0;
 
 
-        if(strlen(p2) <= 2) {
-            reg = hash_item(gMigemoRegsCache, p2);
-
-            if(reg == NULL) {
-                if(gKanjiCode == kUtf8) {
-                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
-                }
-                else if(gKanjiCode == kEucjp) {
-                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
-                }
-                else if(gKanjiCode == kSjis) {
-                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
-                }
-
-                hash_put(gMigemoRegsCache, p2, reg);
-            }
+        if(gKanjiCode == kUtf8) {
+            r = onig_new(&gMigemoReg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
         }
-        else { 
-            if(gKanjiCode == kUtf8) {
-                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
-            }
-            else if(gKanjiCode == kEucjp) {
-                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
-            }
-            else if(gKanjiCode == kSjis) {
-                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
-            }
+        else if(gKanjiCode == kEucjp) {
+            r = onig_new(&gMigemoReg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
+        }
+        else if(gKanjiCode == kSjis) {
+            r = onig_new(&gMigemoReg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
         }
 
         FREE(p2);
@@ -489,8 +438,8 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         migemo_release(gMigemo, (unsigned char*) p);
 
         if(r != ONIG_NORMAL) {
-            onig_free(reg);
-            reg = NULL;
+            onig_free(gMigemoReg);
+            gMigemoReg = NULL;
         }
 
         // ディレクトリのパートを得る
@@ -686,6 +635,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
             string_obj* tmp2 = STRING_NEW("");
             saphire_get_quoted_fname(path, tmp2);
             char* ret = strdup(string_c_str(tmp2));
+            string_delete(tmp2);
 
             FREE(text2);
             return ret;
@@ -700,6 +650,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
                 string_obj* tmp2 = STRING_NEW("");
                 saphire_get_quoted_fname(path, tmp2);
                 char* ret = strdup(string_c_str(tmp2));
+                string_delete(tmp2);
 
                 xstrncpy(path, string_c_str(gEditingDir), PATH_MAX);
                 xstrncat(path, file, PATH_MAX);
@@ -722,7 +673,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         }
         /// migemo 補完 ///
         else {
-            char* ret = readline_file_completion_migemo_match((char*)text2 , file, reg);
+            char* ret = readline_file_completion_migemo_match((char*)text2 , file, gMigemoReg);
             if(ret) {
                 struct stat stat_;
                 char path[PATH_MAX];
@@ -743,9 +694,11 @@ char* readline_file_completion_migemo(const char* text, int rstat)
                 string_obj* tmp = STRING_NEW("");
                 saphire_get_quoted_fname(ret, tmp);
                 free(ret);
+                char* ret =  strdup(string_c_str(tmp));
+                string_delete(tmp);
 
                 FREE(text2);
-                return strdup(string_c_str(tmp));
+                return ret;
             }
         }
     }
@@ -1193,6 +1146,8 @@ char** readline_on_complete(const char* text, int start, int end)
     BOOL user = FALSE;
     BOOL ref = FALSE;
     BOOL method = FALSE;
+    BOOL file = FALSE;
+    vector_obj* commands = VECTOR_NEW(10);
     string_obj* command = STRING_NEW("");
     string_obj* statment = STRING_NEW("");
 
@@ -1208,128 +1163,85 @@ char** readline_on_complete(const char* text, int start, int end)
     BOOL dquote = FALSE;
     while(p < rl_line_buffer + end) {
         // クォート
-        if(*p == '\\' && *(p+1) == 't') {
+        if(*p == '\\') {
+            string_push_back2(statment, *p);
+            string_push_back2(buf, *p);
+            if(program) { string_push_back2(command, *p); }
+            p++;
+            string_push_back2(buf, *p);
             string_push_back2(statment, *p);
-            p++;
-            string_push_back2(statment, *p);
-            p++;
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
-        }
-        else if(*p == '\\' && *(p+1) == 'n') {
-            string_push_back2(statment, *p);
-            p++;
-            string_push_back2(statment, *p);
-            p++;
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
-        }
-        else if(*p == '\\' && *(p+1) == 'r') {
-            string_push_back2(statment, *p);
-            p++;
-            string_push_back2(statment, *p);
-            p++;
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
-        }
-        else if(*p == '\\' && *(p+1) == 'a') {
-            string_push_back2(statment, *p);
-            p++;
-            string_push_back2(statment, *p);
-            p++;
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
-        }
-        else if(*p == '\\') {
-            string_push_back2(statment, *p);
-            p++;
             if(program) { string_push_back2(command, *p); }
-            string_push_back2(statment, *p);
             p++;
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
         }
 
         // シングルクォート
         else if(!dquote && *p == '\'') {
             string_push_back2(statment, *p);
+            string_push_back2(buf, *p);
+            if(program) { string_push_back2(command, *p); }
             p++;
             squote = !squote;
-            user = FALSE;
 
             string_put(buf, "");
         }
         // ダブルクォート
         else if(!squote && *p == '"') {
             string_push_back2(statment, *p);
+            string_push_back2(buf, *p);
+            if(program) { string_push_back2(command, *p); }
             p++;
             dquote = !dquote;
-            user = FALSE;
-            ref = FALSE;
 
             string_put(buf, "");
         }
-        // 環境変数 $$()
-        else if(!squote && *p == '$' && *(p+1) == '$' && *(p+2) == '(') {
-            p+=3;
-
-            program = TRUE;
-            string_put(command, "");
-            string_put(statment, "");
-
-            while(*p == ' ' || *p == '\t' || *p == '\n') {
-                p++;
-            }
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
-        }
-        // 環境変数 $()
-        else if(!squote && *p == '$' && *(p+1) == '(') {
-            p+=2;
-
-            program = TRUE;
-            string_put(command, "");
-            string_put(statment, "");
-            while(*p == ' ' || *p == '\t' || *p == '\n') {
-                p++;
-            }
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
-        }
-        else if(!dquote && !squote && *p == ')') {
-            p++;
-            program = FALSE;
-            while(*p == ' ' || *p == '\t' || *p == '\n') {
-                p++;
-            }
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
-            string_put(statment, "");
-            string_put(command, "");
-        }
 
         /// シングルクォート、ダブルクォート中 ///
         else if(squote || dquote) {
             string_push_back2(statment, *p);
+            string_push_back2(buf, *p);
+            if(program) { string_push_back2(command, *p); }
+            p++;
+        }
+
+        // ブロック、コマンド展開
+        else if(*p == '(' || *p == '{') {
+            string_push_back2(statment, *p);
+            string_push_back2(buf, *p);
+            if(program) { string_push_back2(command, *p); }
             p++;
 
+            while(*p == ' ' || *p == '\t' || *p == '\n') { p++; }
+
+            program = TRUE;
+            user = FALSE;
+            ref = FALSE;
+            method = FALSE;
+            file = FALSE;
+
+            vector_add(commands, command);
+            command = STRING_NEW("");
+
             string_put(buf, "");
+            string_put(statment, "");
+        }
+        // ブロック、コマンド展開終わり
+        else if(*p == ')' || *p == '}') {
+            string_push_back2(statment, *p);
+            string_push_back2(buf, *p);
+            if(program) { string_push_back2(command, *p); }
+            p++;
+
+            program = TRUE;
+            user = FALSE;
+            ref = FALSE;
+            method = FALSE;
+            file = FALSE;
+
+            string_delete(command);
+            command = vector_pop_back(commands);
+
+            string_put(buf, "");
+            string_put(statment, "");
         }
         // リファレンス
         else if(*p == '$') {
@@ -1337,31 +1249,37 @@ char** readline_on_complete(const char* text, int start, int end)
             if(program) { string_push_back2(command, *p); }
             string_push_back2(statment, *p);
             p++;
-            ref = TRUE;
-        }
-        else if(*p == '~') {
-            if(program) { string_push_back2(command, *p); }
-            string_push_back2(statment, *p);
-            p++;
-            ref = FALSE;
-            user = TRUE;
-
-            string_put(buf, "");
-        }
-        /// グロブ ///
-        else if(*p == '*' || *p == '?' || *p =='[' && (*(p+1) != ' ' && *(p+1) != '\t' && *(p+1) == '\n')) 
-        {
-            if(program) { string_push_back2(command, *p); }
-            string_push_back2(statment, *p);
-            p++;
-            ref = FALSE;
+            program = FALSE;
             user = FALSE;
-
-            string_put(buf, "");
+            ref = TRUE;
+            method = FALSE;
+            file = FALSE;
         }
+        /// ユーザー
+        else if(*p == '~') {
+            string_push_back2(buf, *p);
+            if(program) { string_push_back2(command, *p); }
+            string_push_back2(statment, *p);
+            p++;
+
+            program = FALSE;
+            user = TRUE;
+            ref = FALSE;
+            method = FALSE;
+            file = FALSE;
+        }
+        /// メソッド
         else if(*p == '-' && *(p+1) == '>') {
-            p+=2;
-            if(program) { 
+            string_push_back2(buf, *p);
+            if(program) { string_push_back2(command, *p); }
+            string_push_back2(statment, *p);
+            p++;
+            string_push_back2(buf, *p);
+            if(program) { string_push_back2(command, *p); }
+            string_push_back2(statment, *p);
+            p++;
+
+            if(program) {
                 sObject* object;
                 if(string_c_str(buf)[0] == '$') {
                     sRef* ref = saphire_get_ref(string_c_str(buf) + 1);
@@ -1386,26 +1304,36 @@ char** readline_on_complete(const char* text, int start, int end)
                         vector_delete(gMethodCompletions);
                     }
                     gMethodCompletions = saphire_methods_names(object);
+                    program = FALSE;
+                    user = FALSE;
+                    ref = FALSE;
                     method = TRUE; 
-                    program = FALSE;  
+                    file = FALSE;
                 }
                 string_put(buf, "");
             }
-
-            user = FALSE;
-            ref = FALSE;
         }
-        else if(*p == '<' && program) {
+        /// リダイレクト
+        else if(*p == '<' || *p == '>' || *p == '%') {
+            if(program) { string_push_back2(command, *p); }
+            string_push_back2(statment, *p);
             p++;
 
+            while(*p == ' ' || *p == '\t' || *p == '\n'|| *p=='>' || *p == '2') {
+                string_push_back2(statment, *p);
+                if(program) { string_push_back2(command, *p); }
+                p++;
+            }
+
             program = FALSE;
-            ref = FALSE;
             user = FALSE;
+            ref = FALSE;
+            method = FALSE;
+            file = TRUE;
 
             string_put(buf, "");
         }
-        /// 単語の区切り /////////////////////////////////////////////////
-    
+
         /// 空白 ///
         else if(*p == ' ' || *p == '\t') {
             while(*p == ' ' || *p == '\t') {
@@ -1414,29 +1342,13 @@ char** readline_on_complete(const char* text, int start, int end)
             }
 
             program = FALSE;
-            method = FALSE;
             user = FALSE;
             ref = FALSE;
+            method = FALSE;
+            file = FALSE;
 
             string_put(buf, "");
         }
-        /// ブロック
-        else if(*p == '{') {
-            p++;
-            program = TRUE;
-            string_put(command, "");
-            string_put(statment, "");
-            method = FALSE;
-            while(*p == ' ' || *p == '\t' || *p == '\n') {
-                p++;
-            }
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
-        }
-        
-        /// コマンドの区切り ///////////////////////////////////////////
         
         /// パイプ ///
         else if(*p == '|' && *(p+1) != '|') {
@@ -1444,192 +1356,125 @@ char** readline_on_complete(const char* text, int start, int end)
             p++;
 
             program = TRUE;
-            string_put(command, "");
+            user = FALSE;
+            ref = FALSE;
             method = FALSE;
+            file = FALSE;
+
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 string_push_back2(statment, *p);
                 p++;
             }
-            ref = FALSE;
-            user = FALSE;
 
+            string_put(command, "");
             string_put(buf, "");
         }
         
-        /// 文(statment)の区切り系 ///////////////////////////////////
-    
-        /// アンドアンド ///
-        else if(*p == '&' && *(p+1) == '&') {
-            p+=2;
+        /// 論理演算
+        else if(*p == '&' && *(p+1) == '&' || *p == '|' && *(p+1) == '|') {
+            p++;
+            p++;
 
             program = TRUE;
-            string_put(command, "");
-            string_put(statment, "");
-            method = FALSE;
-            while(*p == ' ' || *p == '\t' || *p == '\n') {
-                string_push_back2(statment, *p);
-                p++;
-            }
             user = FALSE;
             ref = FALSE;
-
-            string_put(buf, "");
-        }
-        /// オアオア ///
-        else if(*p == '|' && *(p+1) == '|') {
-            p+=2;
-
-            program = TRUE;
-            string_put(command, "");
-            string_put(statment, "");
             method = FALSE;
+            file = FALSE;
+
             while(*p == ' ' || *p == '\t' || *p == '\n') {
-                string_push_back2(statment, *p);
                 p++;
             }
-            user = FALSE;
-            ref = FALSE;
 
+            string_put(command, "");
+            string_put(statment, "");
             string_put(buf, "");
         }
         /// バックグラウンド ///
-        else if(*p == '&' && *(p+1) != '&') {
+        else if(*p == '&' || *p == ';' || *p == '\n') {
             p++;
+            while(*p == ' ' || *p == '\t' || *p == '\n') { p++; }
 
             program = TRUE;
-            string_put(command, "");
-            string_put(statment, "");
-            method = FALSE;
-            while(*p == ' ' || *p == '\t' || *p == '\n') {
-                string_push_back2(statment, *p);
-                p++;
-            }
             user = FALSE;
             ref = FALSE;
+            method = FALSE;
+            file = FALSE;
 
             string_put(buf, "");
-        }
-        /// セミコロン ///
-        else if(*p == ';') {
-            p++;
-
-            program = TRUE;
             string_put(command, "");
             string_put(statment, "");
-            method = FALSE;
-            while(*p == ' ' || *p == '\t' || *p == '\n') {
-                string_push_back2(statment, *p);
-                p++;
-            }
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
-        }
-        /// 改行 ///
-        else if(*p == '\n') {
-            p++;
-
-            program = TRUE;
-            string_put(command, "");
-            string_put(statment, "");
-            method = FALSE;
-            while(*p == ' ' || *p == '\t' || *p == '\n') {
-                string_push_back2(statment, *p);
-                p++;
-            }
-            user = FALSE;
-            ref = FALSE;
-
-            string_put(buf, "");
         }
         /// コメント ///
         else if(*p == '#') {
             p++;
 
-            while(*p) {
-                if(*p == '\n') {
-                    p++;
-                    program = TRUE;
-                    string_put(command, "");
-                    break;
-                }
-                else {
-                    p++;
-                }
+            while(*p && *p != '\n') {
+                p++;
             }
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
+            program = TRUE;
             user = FALSE;
-            method = FALSE;
             ref = FALSE;
+            method = FALSE;
+            file = FALSE;
 
             string_put(buf, "");
-        }
-        else if(*p == '/') {
-            string_push_back2(buf, *p);
-            user = FALSE;
-            if(program) { string_push_back2(command, *p); }
-            method = FALSE;
-            ref = FALSE;
-            string_push_back2(statment, *p);
-            p++;
+            string_put(command, "");
+            string_put(statment, "");
         }
         /// 終了 ///
         else if(*p == 0) {
             break;
         }
-        
-        /// 構文 ////////////////////////////////////////////////////
 
-        /// サブシェル ///
-        else if(*p == '(') {
-            p++;
-            program = TRUE;
-            string_put(command, "");
-            string_put(statment, "");
-            method = FALSE;
-            user = FALSE;
-            ref = FALSE;
-            while(*p == ' ' || *p == '\t' || *p == '\n') {
-                string_push_back2(statment, *p);
-                p++;
-            }
-
-            string_put(buf, "");
-        }
         else {
             string_push_back2(buf, *p);
-
-            if(program) {
-                string_push_back2(command, *p);
-            }
+            if(program) { string_push_back2(command, *p); }
             string_push_back2(statment, *p);
             p++;
         }
     }
 
+//printf("command %s buf %s\n", string_c_str(command), string_c_str(buf));
+
     sStatments* statments;
     /// 最後の単語入力が=が入っている場合ファイル名の補完をしたいのでカスタマイズ補完は無効にしたい
     if(strstr(string_c_str(buf), "=")) {
         statments = NULL;
     }
-    else if(strcmp(string_c_str(buf), string_c_str(command)) != 0) {
+    else {
         statments = hash_item(gUserCompRegex, string_c_str(command));
     }
-    else {
-        statments = NULL;
-    }
 
+    /// コマンドの補完でもファイル名に/が入っている場合はファイル名の補完にしたい
     if(strstr(string_c_str(command), "/")) {
         program = FALSE;
     }
 
     string_delete(command);
     string_delete(buf);
+    int i;
+    for(i=0; i<vector_size(commands); i++) {
+        string_delete(vector_item(commands, i));
+    }
+    vector_delete(commands);
 
-    if(user) {
+#if defined(HAVE_MIGEMO_H)
+    if(file) {
+        string_delete(statment);
+        char** str = rl_completion_matches(text, readline_file_completion_migemo);
+        if(gMigemoReg) { onig_free(gMigemoReg); gMigemoReg = NULL; }
+        return str;
+    }
+#else
+    if(file) {
+        string_delete(statment);
+        return rl_completion_matches(text, readline_file_completion);
+    }
+#endif
+    else if(user) {
         rl_completion_append_character= 0;
         string_delete(statment);
         return rl_completion_matches(text, rl_username_completion_function);
@@ -1657,7 +1502,9 @@ char** readline_on_complete(const char* text, int start, int end)
 #if defined(HAVE_MIGEMO_H)
     else {
         string_delete(statment);
-        return rl_completion_matches(text, readline_file_completion_migemo);
+        char** str = rl_completion_matches(text, readline_file_completion_migemo);
+        if(gMigemoReg) { onig_free(gMigemoReg); gMigemoReg = NULL; }
+        return str;
     }
 #else
     else {
@@ -1671,14 +1518,12 @@ char** readline_on_complete(const char* text, int start, int end)
 
 static int skip_quoted(const char *s, int i, char q)
 {
-   while(s[i] && s[i]!=q)
+   while(s[i] && s[i]!=q) 
    {
-      if(s[i]=='\\' && s[i+1])
-	 i++;
+      if(s[i]=='\\' && s[i+1]) i++;
       i++;
    }
-   if(s[i])
-      i++;
+   if(s[i]) i++;
    return i;
 }
 
@@ -1697,7 +1542,7 @@ int lftp_char_is_quoted(const char *string,int eindex)
         }
       else if (string[i] == '"' || string[i] == '\'')
         {
-	  char quote = string[i];
+          char quote = string[i];
           i = skip_quoted (string, ++i, quote);
           if (i > eindex)
             return 1;
