@@ -463,6 +463,8 @@ sAtCommand* sAtCommand_new2(sAtCommand* at_command);
 void sAtCommand_delete(sAtCommand* self);
 sAtCommand* sAtCommand_save(sAtCommand* self, int fd);
 sAtCommand* sAtCommand_load(int fd);
+void dig_stack_frame(char* sname, int sline, vector_obj** fun_argv_before, vector_obj* argv2);
+void burry_stack_frame(vector_obj* fun_argv_before, string_obj** sname_strace, int* line_strace);
 
 ////////////////////////////////////////////////////////////
 /// from saphire_commands.c
